from django.db import models
from django.db.models. signals import pre_save
from django.dispatch import receiver
from django.utils import timezone
from wagtail import blocks
from wagtail.admin.panels import (
    TabbedInterface, ObjectList, FieldPanel, InlinePanel, MultiFieldPanel, FieldRowPanel,

    # import PublishingPanel:
    PublishingPanel,
)

from wagtail.fields import StreamField
from wagtail.models import Page,  PreviewableMixin, Orderable, PageManager
from wagtail_trash.managers import TrashManager
from wagtail.search import index
from wagtail.snippets.models import register_snippet, register_deferred_snippets

from taggit.models import TagBase, GenericTaggedItemBase
from taggit.managers import TaggableManager

from modelcluster.fields import ParentalKey
# from django.contrib.auth.models import User
from wagtail.fields import RichTextField
# from django import forms
# # the default Django widgets live here

from modelcluster.models import ClusterableModel
from wagtailmodelchooser import register_model_chooser

import re


# from .abstract_model import BaseBanners
from django import forms
# from .blocks import WebMultiColumnStreamBlock, HFStreamBlock
from django.core.validators import MaxValueValidator, MinValueValidator
from wagtail.models import PreviewableMixin

from .blocks import *
# from django_extensions.db.fields import AutoSlugField
from wagtail.documents import get_document_model
from django.db.models.signals import pre_save
from django.dispatch import receiver
from django.template.defaultfilters import slugify

import uuid


@register_snippet
@register_model_chooser
class CompanyInfo(models.Model):
    ciType = models.CharField(max_length=200, null=True, choices=[
        ('Name', 'Name'),
        ('Address', 'Address'),
        ('Email', 'Email'),
        ('Contact', 'Contact'),
        ('Logo', 'Logo')
    ], verbose_name="Info Type")
    ciName = models.CharField(
        max_length=200, null=True, blank=True, verbose_name="Name")
    ciTitle = models.CharField(
        max_length=200, null=True, blank=True, verbose_name="Title")
    ciLogo = models.ForeignKey(
        "wagtailimages.Image",
        null=True,
        blank=True,
        on_delete=models.SET_NULL,
        related_name="+",
        verbose_name="Logo"
    )
    ciLogoText = models.CharField(
        max_length=250, blank=True, null=True, verbose_name="Logo Text")
    ciBody = StreamField([
        ('text', CharBlock()),
        ('paragraph', RichTextBlock()),
    ], blank=True, null=True, verbose_name="Body")

    # modify your content_panels:
    panels = [
        MultiFieldPanel([
            FieldRowPanel([
                FieldPanel('ciType'),
                FieldPanel('ciName'),

                FieldPanel('ciTitle'),

            ]),
            FieldPanel('ciLogo'),
            FieldPanel('ciLogoText'),
        ]),
        FieldPanel('ciBody'),
    ]

    class Meta:
        verbose_name_plural = "Company Info"
        ordering = ['ciTitle']

    def __str__(self):
        if self.ciTitle and self.ciName:
            return f'{self.ciName}  ::  {self.ciTitle}  :: {self.ciType}'
        if self.ciTitle:
            return f'{self.ciTitle}  :: {self.ciType}'
        else:
            return f'{self.ciType}'


@register_model_chooser
@register_snippet
class Social(Orderable):
    soTitle = models.CharField(blank=True, null=True,
                               max_length=255,
                               help_text="Social Plateform Name e.g: Facebook,Instagram",
                               verbose_name="Title"
                               )
    soLogo = models.ForeignKey(
        "wagtailimages.Image",
        null=True,
        on_delete=models.SET_NULL,
        related_name="+",
        verbose_name="Icon"
    )
    soLogoText = models.CharField(
        max_length=250, blank=True, null=True, verbose_name="Logo Text")

    soUrl = models.CharField(
        null=True,
        verbose_name="URL",
        max_length=255,
        help_text="For Whatsapp use https://wa.me/8859XXXXXX",
    )
    panels = [
        MultiFieldPanel([
            FieldRowPanel(
                [
                    FieldPanel("soTitle"),
                    FieldPanel("soLogo"),

                ]
            ),
            FieldPanel("soUrl"),
        ]),
    ]

    def __str__(self):
        return f'{self.soTitle}'

    class Meta:
        verbose_name_plural = "Social"
        ordering = ['soTitle']


class WebPage(Page):
    objects = PageManager()  # needed, so _default_manager isn't the trash manager
    objects_excluding_trash = TrashManager()
    body = StreamField([
        ('carousel', CarouselBlock()),
        ('section', SectionBlock()),
        ('row', RowBlock()),

    ])
    show_footer = models.BooleanField(blank=True, null=True, default=False)

    # modify your content_panels:
    content_panels = Page.content_panels + [
        FieldPanel('body'),
        FieldPanel('show_footer')
    ]

    def breadcrumb_list(self):
        breadcrumbs = []
        page = self
        while page.get_parent() is not None:
            if page.is_root():
                break
            page = page.get_parent()
            breadcrumbs.append(page)
        breadcrumbs.reverse()
        return breadcrumbs


class CategoryIndexPage(Page):
    objects = PageManager()  # needed, so _default_manager isn't the trash manager
    objects_excluding_trash = TrashManager()
    intro = RichTextField(blank=True)
    show_footer = models.BooleanField(blank=True, null=True, default=True)
    banner = StreamField([
        ('banner', CarouselBlock()),
    ], use_json_field=True, blank=True, null=True, max_num=1)
    # parent_page_type = [
    #     'core.WebPages',
    # ]
    subpage_types = [
        'com.CategoryPage',  # appname.ModelName
    ]
    content_panels = Page.content_panels + [
        FieldPanel('intro'),
        # FieldPanel('banner')
    ]

    def get_children(self):
        # qs = super().get_children()
        qs = CategoryPage.objects.child_of(self)
        qs = qs.order_by('order')
        return qs

 

    def get_child_categories(self):
        qs = CategoryPage.objects.child_of(self)
        qs = qs.order_by('order')
        return qs

    def breadcrumb_list(self):
        breadcrumbs = []
        page = self
        while page.get_parent() is not None:
            if page.is_root():
                break
            page = page.get_parent()
            breadcrumbs.append(page)
        breadcrumbs.reverse()
        return breadcrumbs

    def mobile_breadcrumb_list(self):
        breadcrumbs = []
        page = self
        while page.get_parent() is not None:
            if page.is_root():
                break
            page = page.get_parent()
            breadcrumbs.append(page)
        breadcrumbs.reverse()
        return breadcrumbs[3:]
    # def get_categories(self):
    #     return CategoryPage.objects.filter(type="Category").order_by('-id')


class CategoryPage(Page):
    objects = PageManager()  # needed, so _default_manager isn't the trash manager
    objects_excluding_trash = TrashManager()
    catMenu = models.CharField(
        max_length=100, null=True, blank=True, verbose_name="Menu Name")
    catImage = models.ForeignKey(
        'wagtailimages.Image', on_delete=models.SET_NULL, related_name='+', blank=True, null=True, verbose_name="Category Image"
    )
    show_footer = models.BooleanField(blank=True, null=True, default=True)
    order = models.PositiveBigIntegerField(blank=True, null=True)
    subpage_types = [
        'com.CategoryPage',  # appname.ModelName
        'com.ProductPage',  # appname.ModelName
    ]

    content_panels = Page.content_panels + [
        FieldPanel('order'),
        FieldPanel('catMenu'),
        FieldPanel('catImage'),
    ]

    def get_children(self):
        # qs = super().get_children()
        # qs = qs.order_by('order')
        if CategoryPage.objects.child_of(self):
            qs = CategoryPage.objects.child_of(self)
        else:
            qs = ProductPage.objects.child_of(self)
        qs = qs.order_by('order')
        return qs
   

    def breadcrumb_list(self):
        breadcrumbs = []
        page = self
        while page.get_parent() is not None:
            if page.is_root():
                break
            page = page.get_parent()
            breadcrumbs.append(page)
        breadcrumbs.reverse()
        return breadcrumbs

    def mobile_breadcrumb_list(self):
        breadcrumbs = []
        page = self
        while page.get_parent() is not None:
            if page.is_root():
                break
            page = page.get_parent()
            breadcrumbs.append(page)
        breadcrumbs.reverse()
        return breadcrumbs[4:]

    class Meta:
        ordering = ['order']
        verbose_name = "Category"


class ProductPage(Page):
    objects = PageManager()  # needed, so _default_manager isn't the trash manager
    objects_excluding_trash = TrashManager()
    prImage = models.ForeignKey(
        "wagtailimages.Image",
        null=True,
        blank=True,
        on_delete=models.SET_NULL,
        related_name="+",
        verbose_name="Image"
    )
    prImages = StreamField([
        ('image', ImageChooserBlock()),
    ], blank=True, null=True, verbose_name="Images")

    prDatasheet = models.ForeignKey(
        get_document_model(),
        null=True,
        blank=True,
        on_delete=models.SET_NULL,
        verbose_name="PDF DataSheet"
    )

    body = StreamField([
        ('carousel', CarouselBlock()),
        ('section', SectionBlock()),
        ('row', RowBlock()),

    ], null=True, blank=True, use_json_field=True)

    prDesc = RichTextField(blank=True, null=True,
                           verbose_name="Description")
    show_footer = models.BooleanField(blank=True, null=True, default=True)
    order = models.PositiveBigIntegerField(blank=True, null=True)

    # modify your content_panels:
    content_panels = Page.content_panels + [
        MultiFieldPanel([
            FieldRowPanel([
                FieldPanel('order'),
                FieldPanel('prImage'),
                FieldPanel('prDatasheet'),

            ]),

            FieldPanel('prDesc'),
        ]),
    ]

    def breadcrumb_list(self):
        breadcrumbs = []
        page = self
        while page.get_parent() is not None:
            if page.is_root():
                break
            page = page.get_parent()
            breadcrumbs.append(page)
        breadcrumbs.reverse()
        return breadcrumbs

    def mobile_breadcrumb_list(self):
        breadcrumbs = []
        page = self
        while page.get_parent() is not None:
            if page.is_root():
                break
            page = page.get_parent()
            breadcrumbs.append(page)
        breadcrumbs.reverse()
        return breadcrumbs[5:]

    class Meta:
        ordering = ('order',)
        verbose_name = "Product"


# @register_snippet
class Enquiry(models.Model):
    id = models.UUIDField(unique=True, primary_key=True, editable=False,
                          default=uuid.uuid4, verbose_name='Public identifier')
    enqName = models.CharField(max_length=100, null=True, blank=True,verbose_name="Name")
    enqEmail = models.EmailField(null=True, blank=True)
    enqContact = models.CharField(max_length=20, null=True, blank=True,verbose_name="Contact")
    enqCompany = models.CharField(max_length=100, null=True, blank=True)
    enqMessage = models.TextField(null=True, blank=True)
    enqCreatedOn = models.DateTimeField(auto_now_add=True,verbose_name="Date")
    enqStatus = models.CharField(
        max_length=100, null=True, blank=True, default="Pending")
    parent = models.ForeignKey(
        'self', on_delete=models.CASCADE, null=True, blank=True, related_name='replies')
    # enqProduct = models.ForeignKey(
    #     Products, on_delete=models.CASCADE, null=True, blank=True)
    enqProduct = models.CharField(max_length=200, null=True, blank=True)

    def __str__(self):
        if self.parent == None:
            return f'{self.enqName}'
        else:
            return f'{self.parent}  => {self.enqName}'

    class Meta:
        ordering = ['-enqCreatedOn']


@register_snippet
class FollowUp(models.Model):
    id = models.UUIDField(unique=True, primary_key=True, editable=False,
                          default=uuid.uuid4, verbose_name='Public identifier')
    enquiry = models.ForeignKey(
        Enquiry, on_delete=models.CASCADE, null=True, blank=True)
    detail = RichTextField(null=True, blank=True)
    assignTo = models.CharField(max_length=200, null=True, blank=True)
    remarks = models.TextField(null=True, blank=True)
    enqCreatedOn = models.DateTimeField(auto_now_add=True)
    parent = models.ForeignKey(
        'self', on_delete=models.CASCADE, null=True, blank=True, related_name='replie_follow')

    def __str__(self):
        if self.parent == None:
            return f'{self.assignTo}'
        else:
            return f'{self.enquiry}  => {self.parent} => {self.assignTo}'

    class Meta:
        ordering = ['-enqCreatedOn']
